<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Catalog\Model\ResourceModel\Product\Action as ProductAction;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Ui\Component\MassAction\Filter;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Catalog\Api\ProductRepositoryInterface;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use WriteTextAI\WriteTextAI\Model\ReviewStatus;
use Magento\Store\Model\Store;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;

class MassTransfer extends Action implements HttpPostActionInterface
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::transfer';

    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var Filter
     */
    protected $filter;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var ProductAction
     */
    protected $productAction;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var ReviewStatus
     */
    protected $reviewStatus;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param GeneratedText $generatedText
     * @param Filter $filter
     * @param CollectionFactory $collectionFactory
     * @param ProductAction $productAction
     * @param Data $helper
     * @param AiProductManager $aiProductManager
     * @param Premium $premium
     * @param ApiManager $apiManager
     * @param ProductRepositoryInterface $productRepository
     * @param ProductHelper $productHelper
     * @param ReviewStatus $reviewStatus
     * @param StoreHelper $storeHelper
     */
    public function __construct(
        Context $context,
        GeneratedText $generatedText,
        Filter $filter,
        CollectionFactory $collectionFactory,
        ProductAction $productAction,
        Data $helper,
        AiProductManager $aiProductManager,
        Premium $premium,
        ApiManager $apiManager,
        ProductRepositoryInterface $productRepository,
        ProductHelper $productHelper,
        ReviewStatus $reviewStatus,
        StoreHelper $storeHelper
    ) {
        $this->generatedText = $generatedText;
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        $this->productAction = $productAction;
        $this->helper = $helper;
        $this->aiProductManager = $aiProductManager;
        $this->premium = $premium;
        $this->apiManager = $apiManager;
        $this->productRepository = $productRepository;
        $this->productHelper = $productHelper;
        $this->reviewStatus = $reviewStatus;
        $this->storeHelper = $storeHelper;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $result = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $result->setData([
                'success' => false,
                'reload' => true,
                'message' => __('Premium is required to do this action.')
            ]);

            return $result;
        }
        
        try {
            $fields = $this->getRequest()->getPost('fields', []);
            $stores = $this->getRequest()->getPost('stores', []);
            $filters = $this->getRequest()->getPost('filters', []);

            $storeFilter = isset($filters['store_id']) ?
                $filters['store_id'] : Store::DEFAULT_STORE_ID;
            $collection = $this->filter->getCollection($this->collectionFactory
                ->create()
                ->addAttributeToSelect('*'))
                ->setStoreId($storeFilter)
                ->addMediaGalleryData();
            $resultData = [
                'success' => false,
                'message' => __(
                    'Transfer failed because there are no generated ' .
                    'texts yet for some of the products selected. ' .
                    'Generate text first before transferring.'
                )
            ];
            
            foreach ($collection as $product) {
                $productId = $product->getId();
                $item = $this->generatedText->getTextFields($productId, $storeFilter, true);
                $productFields = $this->removeEmptyAndTransferredFields($item, $fields);

                if (!empty($productFields)) {
                    $resultData = [
                        'success' => true,
                        'fields' => $productFields
                    ];

                    $this->generatedText->saveText(
                        $storeFilter,
                        $productId,
                        $item['page_title'],
                        $item['page_description'],
                        $item['product_description'],
                        $item['short_product_description'],
                        $item['open_graph'],
                        $productFields
                    );
    
                    foreach ($stores as $storeId) {
                        $this->transfer(
                            $productId,
                            (int)$storeId,
                            $productFields,
                            $item['page_title'],
                            $item['page_description'],
                            $item['product_description'],
                            $item['short_product_description'],
                            $item['open_graph']
                        );
                    }
                }
            
                if (in_array('alt_text', $fields)) {
                    $resultData = [
                        'success' => true
                    ];
                    $this->transferImageAltTexts(
                        $productId,
                        $stores,
                        $storeFilter,
                        $product
                    );
                }

                $this->reviewStatus->updateReview($productFields, $productId, $stores);
            }

            $result->setData($resultData);
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if (!$message) {
                $message = __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                );
            }

            $result->setData([
                'success' => false,
                'message' => $message
            ]);
        }

        return $result;
    }

    /**
     * Transfer image alt texts
     *
     * @param int $productId
     * @param array $selectedStores
     * @param int $storeFilter
     * @param \Magento\Catalog\Model\Product $product
     * @return void
     */
    private function transferImageAltTexts($productId, $selectedStores, $storeFilter, $product)
    {
        $language = $this->storeHelper->getFormattedLanguage($storeFilter);
        $images = $this->productHelper->getImages($product, $storeFilter, $language);

        $imagesToUpdate = [];
        foreach ($selectedStores as $storeId) {
            $product = $this->productRepository->getById($productId, true, $storeId);
            $mediaGalleryEntries = $product->getMediaGalleryEntries();
            $language = $this->storeHelper->getFormattedLanguage($storeId);
    
            foreach ($images as $image) {
                foreach ($mediaGalleryEntries as $mediaGalleryEntry) {
                    if ($mediaGalleryEntry->getId() == $image['id']) {
                        $mediaGalleryEntry->setLabel($image['writetext_alt']);
                        break;
                    }
                }

                try {
                    $getParams = [
                        "storeId" => $storeId,
                        "language" => $language,
                        "imageId" => $image['id']
                    ];

                    $altText = $this->apiManager->getImage($getParams);

                    if (isset($altText['altText']['id'])) {
                        $imagesToUpdate[] = [
                            "storeId" => $storeId,
                            "language" => $language,
                            "imageId" => $altText['imageId'],
                            "textId" => $altText['altText']['id'],
                            "value" => $image['writetext_alt'],
                            "publish" => true,
                            "reviewed" => true
                        ];
                    }
                } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
                    if ($e->getCode() !== 404) {
                        throw $e;
                    }
                }
            }

            $product->setMediaGalleryEntries($mediaGalleryEntries);

            $this->productRepository->save($product);
            
            $this->aiProductManager->saveDate($productId, $storeId, 'transferred');
        }

        $this->apiManager->batchUpdateAltText(json_encode([
            "images" => $imagesToUpdate
        ]));
    }

    /**
     * Remove empty and already transferred fields
     *
     * @param array $item
     * @param array $fields
     * @return array
     */
    private function removeEmptyAndTransferredFields($item, $fields)
    {
        $productFields = [];

        foreach ($fields as $field) {
            if (isset($item[$field]) &&
                $item[$field] != '' &&
                $item[$field] != null
            ) {
                $productFields[] = $field;
            }
        }

        return $productFields;
    }

    /**
     * Transfer
     *
     * @param int $productId
     * @param int $storeId
     * @param array $selectedFields
     * @param string $pageTitle
     * @param string $pageDescription
     * @param string $productDescription
     * @param string $productShortDescription
     * @param string $openGraph
     * @return void
     */
    private function transfer(
        $productId,
        $storeId,
        $selectedFields,
        $pageTitle,
        $pageDescription,
        $productDescription,
        $productShortDescription,
        $openGraph
    ) {
        $mappingSettings = $this->helper->getMappingSettings();
        
        $productDescription = $this->productHelper->convertNewLinesToBr($productDescription);
        $productShortDescription = $this->productHelper->convertNewLinesToBr($productShortDescription);

        $fields = [
            'page_title' => $pageTitle,
            'page_description' => $pageDescription,
            'product_description' => $productDescription,
            'short_product_description' => $productShortDescription,
            'open_graph' => $openGraph
        ];
        
        $toUpdate = [];
        foreach ($fields as $field => $value) {
            if (isset($mappingSettings[$field]) && in_array($field, $selectedFields)) {
                $toUpdate[$mappingSettings[$field]] = $value;
            }
        }

        $this->productAction->updateAttributes(
            [$productId],
            $toUpdate,
            $storeId
        );

        $this->aiProductManager->saveDate($productId, $storeId, 'transferred');
    }
}
