<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Ui\Component\MassAction\Filter;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Store\Model\Store;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use Magento\Catalog\Api\ProductRepositoryInterface;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;

class Transferable extends Action implements HttpPostActionInterface
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::transfer';

    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var Filter
     */
    protected $filter;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;

    /**
     * @var array
     */
    protected $products = [];

    /**
     * Constructor
     *
     * @param Context $context
     * @param GeneratedText $generatedText
     * @param Filter $filter
     * @param CollectionFactory $collectionFactory
     * @param Premium $premium
     * @param ProductHelper $productHelper
     * @param ApiManager $apiManager
     * @param ProductRepositoryInterface $productRepository
     * @param StoreHelper $storeHelper
     */
    public function __construct(
        Context $context,
        GeneratedText $generatedText,
        Filter $filter,
        CollectionFactory $collectionFactory,
        Premium $premium,
        ProductHelper $productHelper,
        ApiManager $apiManager,
        ProductRepositoryInterface $productRepository,
        StoreHelper $storeHelper
    ) {
        $this->generatedText = $generatedText;
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        $this->premium = $premium;
        $this->productHelper = $productHelper;
        $this->apiManager = $apiManager;
        $this->productRepository = $productRepository;
        $this->storeHelper = $storeHelper;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $result = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $result->setData([
                'success' => false,
                'reload' => true,
                'message' => __('Premium is required to do this action.')
            ]);

            return $result;
        }
        
        try {
            $fields = $this->getRequest()->getPost('fields', []);
            $filters = $this->getRequest()->getPost('filters', []);

            $storeFilter = isset($filters['store_id']) ? $filters['store_id'] : Store::DEFAULT_STORE_ID;
            $collection = $this->filter->getCollection($this->collectionFactory
                ->create()
                ->addAttributeToSelect('*'))
                ->setStoreId($storeFilter)
                ->addMediaGalleryData();
            $resultData = [
                'success' => false,
                'message' => __(
                    'Transfer failed because there are no generated ' .
                    'texts yet for some of the products selected. ' .
                    'Generate text first before transferring.'
                )
            ];

            $this->collectProducts($collection, $storeFilter);
            
            foreach ($collection as $product) {
                $item = $this->generatedText->getTextFields($product->getId(), $storeFilter, true);
                $productFields = $this->removeEmptyAndTransferredFields(
                    $item,
                    $fields,
                    $product->getId(),
                    $storeFilter
                );
                
                if (!empty($productFields)) {
                    $resultData = [
                        'success' => true
                    ];

                    break;
                }
            }

            $result->setData($resultData);
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if (!$message) {
                $message = __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                );
            }

            $result->setData([
                'success' => false,
                'message' => $message
            ]);
        }

        return $result;
    }

    /**
     * Collect products
     *
     * @param \Magento\Catalog\Model\ResourceModel\Product\Collection $collection
     * @param int $storeId
     * @return void
     */
    private function collectProducts($collection, $storeId)
    {
        $this->products = [];

        foreach ($collection as $product) {
            $this->products[$product->getId()] = $product;
        }
    }

    /**
     * Remove empty and already transferred fields
     *
     * @param array $item
     * @param array $fields
     * @param int $productId
     * @param int $storeFilter
     * @return array
     */
    private function removeEmptyAndTransferredFields($item, $fields, $productId, $storeFilter)
    {
        $productFields = [];

        foreach ($fields as $field) {
            if (isset($item[$field]) &&
                $item[$field] != '' &&
                $item[$field] != null
            ) {
                $productFields[] = $field;
            }
        }

        if (in_array('alt_text', $fields) && $this->isImageGenerated($productId, $storeFilter)) {
            $productFields[] = 'alt_text';
        }

        return $productFields;
    }

    /**
     * Check if image is generated
     *
     * @param int $productId
     * @param int $storeId
     * @return bool
     */
    private function isImageGenerated($productId, $storeId)
    {
        $generated = false;

        $language = $this->storeHelper->getFormattedLanguage($storeId);

        if (isset($this->products[$productId])) {
            $product = $this->products[$productId];
        } else {
            $product = $this->productRepository->getById($productId, false, $storeId);
        }

        try {
            $images = $this->productHelper->getImages($product, $storeId, $language);
        } catch (\Exception $e) {
            return false;
        }

        $mediaGalleryEntries = $product->getMediaGalleryEntries();
 
        foreach ($images as $image) {
            foreach ($mediaGalleryEntries as $mediaGalleryEntry) {
                if ($mediaGalleryEntry->getId() == $image['id']) {
                    $mediaGalleryEntry->setLabel($image['writetext_alt']);
                    break;
                }
            }

            try {
                $getParams = [
                    "storeId" => $storeId,
                    "language" => $language,
                    "imageId" => $image['id']
                ];

                $altText = $this->apiManager->getImage($getParams);

                if (isset($altText['altText']['id'])) {
                    $generated = true;
                    break;
                }
            } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
                if ($e->getCode() !== 404) {
                    throw $e;
                }
            }
            
            if ($generated) {
                break;
            }
        }

        return $generated;
    }
}
