<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Backend\Model\UrlInterface;
use Magento\Store\Model\Store;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\Helper\Image as WtaImageHelper;
use Magento\Catalog\Helper\Image as ImageHelper;
use Magento\Framework\Exception\FileSystemException;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory as ProductCollectionFactory;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;

class UploadThumbnails extends Action
{
    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var WtaImageHelper
     */
    protected $wtaImageHelper;

    /**
     * @var ImageHelper
     */
    protected $imageHelper;

    /**
     * @var ProductCollectionFactory
     */
    protected $productCollectionFactory;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;

    /**
     * @var array
     */
    protected $products = [];

    /**
     * Constructor
     *
     * @param Context $context
     * @param UrlInterface $urlBuilder
     * @param ProductRepositoryInterface $productRepository
     * @param RemoteAddress $remoteAddress
     * @param WtaImageHelper $wtaImageHelper
     * @param ImageHelper $imageHelper
     * @param ProductCollectionFactory $productCollectionFactory
     * @param StoreHelper $storeHelper
     */
    public function __construct(
        Context $context,
        UrlInterface $urlBuilder,
        ProductRepositoryInterface $productRepository,
        RemoteAddress $remoteAddress,
        WtaImageHelper $wtaImageHelper,
        ImageHelper $imageHelper,
        ProductCollectionFactory $productCollectionFactory,
        StoreHelper $storeHelper
    ) {
        $this->urlBuilder = $urlBuilder;
        $this->productRepository = $productRepository;
        $this->remoteAddress = $remoteAddress;
        $this->wtaImageHelper = $wtaImageHelper;
        $this->imageHelper = $imageHelper;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->storeHelper = $storeHelper;

        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $uploadResult = $this->uploadThumbnails();
            $result = [
                'success' => true,
                'failed_thumbnails' => $uploadResult['failed_thumbnails'] ?? [],
                'failed_products' => $uploadResult['failed_products'] ?? [],
                'success_thumbnails' => $uploadResult['success_thumbnails'] ?? []
            ];
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            $result = [
                'success' => false,
                'unauthorized' => true,
                'login_url' => $this->urlBuilder->getUrl('wtai/setup/index'),
                'message' => $e->getMessage()
            ];
        } catch (\Exception $e) {
            $message = $e->getMessage();
            if (!$message) {
                $message = __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                );
            }

            $result = [
                'success' => false,
                'message' => $message
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Upload thumbnails
     */
    protected function uploadThumbnails()
    {
        $selected = $this->getRequest()->getParam('selected', []);
        $storeId = $this->getRequest()->getParam('store_id', Store::DEFAULT_STORE_ID);
        $language = $this->storeHelper->getFormattedLanguage($storeId);
        $ipAddress = $this->remoteAddress->getRemoteAddress();

        $failedThumbnails = [];
        $failedProducts = [];
        $successThumbnails = [];

        $this->collectProducts($selected, $storeId);

        foreach ($selected as $productId) {
            if (isset($this->products[$productId])) {
                $product = $this->products[$productId];
            } else {
                $product = $this->productRepository->getById($productId, false, $storeId);
            }

            $thumbnailId = $this->wtaImageHelper->getImageValue($product);
            if (empty($thumbnailId)) {
                continue;
            }

            $imageHelper = $this->imageHelper->init($product, 'product_listing_thumbnail');
            try {
                $thumbnail = $imageHelper->setImageFile($product->getData('thumbnail'));
                $thumbnailUrl = $thumbnail->getUrl();
                $thumbnail->getOriginalWidth();
            } catch (\InvalidArgumentException | FileSystemException $e) {
                continue;
            }

            try {
                $successThumbnails[] = $this->wtaImageHelper->getImageApiId(
                    $storeId,
                    $ipAddress,
                    $language,
                    $thumbnailId,
                    $thumbnailUrl
                );
            } catch (\WriteTextAI\WriteTextAI\Exception\ImageUnsupported $e) {
                $failedProducts[] = $productId;
                $failedThumbnails[] = $thumbnailUrl;
            } catch (\WriteTextAI\WriteTextAI\Exception\ImageInvalid $e) {
                $failedProducts[] = $productId;
                $failedThumbnails[] = $thumbnailUrl;
            }
        }

        return [
            'failed_thumbnails' => $failedThumbnails,
            'failed_products' => $failedProducts,
            'success_thumbnails' => $successThumbnails
        ];
    }

    /**
     * Collect products
     *
     * @param array $productIds
     * @param int $storeId
     * @return void
     */
    private function collectProducts($productIds, $storeId)
    {
        $this->products = [];

        $collection = $this->productCollectionFactory->create();
        $collection->setStoreId($storeId);
        $collection->addAttributeToSelect('thumbnail');
        $collection->addFieldToFilter('entity_id', ['in' => $productIds])->addMediaGalleryData();

        foreach ($collection as $product) {
            $this->products[$product->getId()] = $product;
        }
    }
}
