<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

class Data extends Settings
{
    /**
     * Get API auth Url
     *
     * @return string
     */
    public function getApiAuthUrl()
    {
        return $this->getApiSettings('auth_url');
    }
    
    /**
     * Get API web Url
     *
     * @return string
     */
    public function getApiWebUrl()
    {
        return $this->getApiSettings('web_url');
    }
    
    /**
     * Get Signalr Url
     *
     * @return string
     */
    public function getSignalrUrl()
    {
        $signalRUrl =  $this->getApiSettings('signalr_url');
        $regionUrl = (string) $this->getApiSettings('region');
        if ($regionUrl) {
            $signalRUrl = sprintf($signalRUrl, $regionUrl);
        }
        return $signalRUrl;
    }

    /**
     * Get API region Url
     *
     * @return string
     */
    public function getRegion()
    {
        $region = (string) $this->getApiSettings('region');
        $regionUrl = $region;
        if (substr($regionUrl, 0, 4) !== "http") {
            $regionUrl = "https://{$region}/";
        }

        return $regionUrl;
    }

    /**
     * Get API client id
     *
     * @return string
     */
    public function getClientId()
    {
        return $this->getApiSettings('client_id');
    }

    /**
     * Get account token
     *
     * @return string
     */
    public function getAccountToken()
    {
        return $this->getApiSettings('account_token');
    }

    /**
     * Check if logger is enabled
     *
     * @return bool
     */
    public function isLoggerEnabled()
    {
        return (bool)$this->getApiSettings('enable_logger');
    }

    /**
     * Get tones
     *
     * @return string
     */
    public function getTones()
    {
        return $this->getGeneralSettings('tones');
    }

    /**
     * Get styles
     *
     * @return string
     */
    public function getStyles()
    {
        return $this->getGeneralSettings('styles');
    }

    /**
     * Get audiences
     *
     * @return string
     */
    public function getAudiences()
    {
        return $this->getGeneralSettings('audiences');
    }

    /**
     * Is translation notice enabled
     */
    public function isTranslationNoticeEnabled()
    {
        return (bool)$this->getTranslationSettings('notice');
    }

    /**
     * Set account token
     *
     * @param string $token
     *
     * @return void
     */
    public function setAccountToken($token)
    {
        $this->setConfig(self::API_SETTINGS_CONFIG_PATH . 'account_token', $token);
    }

    /**
     * Set region
     *
     * @param string $region
     *
     * @return void
     */
    public function setRegion($region)
    {
        $this->setConfig(self::API_SETTINGS_CONFIG_PATH . 'region', $region);
    }

    /**
     * Check if custom open graph is enabled
     *
     * @return bool
     */
    public function getCustomOpenGraph()
    {
        return (bool)$this->getMappingSettings('custom_open_graph');
    }
    
    /**
     * Get data mapping fields
     *
     * @return array
     */
    public function getDataMappingFields()
    {
        return [
            'page_title'                => __('Product meta title'),
            'page_description'          => __('Product meta description'),
            'product_description'       => __('Product description'),
            'short_product_description' => __('Product short description'),
            'open_graph'                => __('Product Open Graph')
        ];
    }

    /**
     * Get data mapping fields
     *
     * @return array
     */
    public function getCategoryDataMappingFields()
    {
        return [
            'category_description'      => __('Category description'),
        ];
    }

    /**
     * Get text type attributes
     *
     * @return array
     */
    public function getTextTypeAttributes()
    {
        return $this->textTypeAttributes->toOptionArray();
    }

       /**
        * Get text type attributes
        *
        * @return array
        */
    public function getCateoryTextTypeAttributes()
    {
        $data = $this->categoryTextTypeAttributes->toOptionArray();
        return $data;
    }

    /**
     * Check if all text fields are mapped to an existing attribute
     *
     * @return boolean
     */
    public function isDataMapped()
    {
        if (empty($this->getMappingSettings())) {
            return false;
        }
        
        $attributes = array_column($this->getTextTypeAttributes(), 'value');
        foreach ($this->getMappingSettings() as $field => $attribute) {
            if ($field === 'open_graph' && !$this->getCustomOpenGraph()) {
                continue;
            }
            if (!$attribute || !in_array($attribute, $attributes)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Get not mapped field
     *
     * @return string
     */
    public function getNotMappedField()
    {
        // Set first field as not mapped field in case all field is not mapped
        $notMappedField = 'page_title';

        if (empty($this->getMappingSettings())) {
            return $this->getFieldLabel($notMappedField);
        }

        $attributes = array_column($this->getTextTypeAttributes(), 'value');
        foreach ($this->getMappingSettings() as $field => $attribute) {
            if ($field === 'open_graph' && !$this->getCustomOpenGraph()) {
                continue;
            }
            if (!$attribute || !in_array($attribute, $attributes)) {
                $notMappedField = $field;
            }
        }

        return $this->getFieldLabel($notMappedField);
    }

    /**
     * Get field label
     *
     * @param string $field
     * @return string
     */
    public function getFieldLabel($field)
    {
        switch ($field) {
            case 'page_title':
                return __('Meta title');
            case 'page_description':
                return __('Meta description');
            case 'product_description':
                return __('Description');
            case 'short_product_description':
                return __('Short description');
            case 'open_graph':
                return __('Open Graph text');
        }
    }

    /**
     * Check if final step is done
     *
     * @return boolean
     */
    public function isFinalStepDone()
    {
        $keywordOptimizationFields = $this->getKeywordOptimizationSettings();

        $generalFields = $this->getGeneralSettings();
        if (empty($generalFields) || empty($keywordOptimizationFields)) {
            return false;
        }

        $keywordMergedArray = array_merge($keywordOptimizationFields, $keywordOptimizationFields['pipeline_keyword']);
        unset($keywordMergedArray['pipeline_keyword']);
        
        $requiredFields = [
            'tones',
            'styles',
        ];

        foreach ($generalFields as $field => $value) {
            if (empty($value) && in_array($field, $requiredFields)) {
                return false;
            }
        }

        $requiredKeywords = [
            'cluster_based_pipelines_count',
            'notification',
            'trigger_action',
            'trigger_action_n_position',
            'trigger_action_n_days',
            'automatic_text_optimization',
            'products',
            'categories',
            'trigger_action_n_days',
            'trigger_action_n_position'
        ];
        
        foreach ($keywordMergedArray as $field => $value) {
            if (empty($value) && in_array($field, $requiredKeywords)) {
                if ($field === 'trigger_action_n_position') {
                    if ($keywordMergedArray['trigger_action'] !== 'when_page_is_ranking') {
                        continue;
                    }
                }
                if ($field === 'trigger_action_n_days') {
                    if ($keywordMergedArray['trigger_action'] !== 'after_n_days') {
                        continue;
                    }
                }
                if ($field === 'products' ||
                    $field === 'categories' ||
                    $field === 'automatic_text_optimization') {
                    if ($keywordMergedArray['trigger_action'] === 'none') {
                        continue;
                    }
                }
                if ($field === 'cluster_based_pipelines_count') {
                    if ($value === '0') {
                        continue;
                    }
                }
                return false;
            }
        }

        return true;
    }

    /**
     * Check if setup is finalized
     *
     * @return bool
     */
    public function isSetupFinalized()
    {
        return $this->getSetupSettings('finalized');
    }

    /**
     * Get reference product collection
     *
     * @param int $storeId
     * @param int $productId
     * @param string $search
     * @param int $page
     * @param int $pageSize
     * @return array
     */
    public function getReferenceProductCollection(
        $storeId = \Magento\Store\Model\Store::DEFAULT_STORE_ID,
        $productId = null,
        $search = '',
        $page = 1,
        $pageSize = 50
    ) {
        $mappingSettings = $this->getMappingSettings();
        $collection = $this->productCollectionFactory->create();
        $collection->setStoreId($storeId);

        $fieldsToSelect = [
            'id',
            'name',
            $mappingSettings['page_title'],
            $mappingSettings['page_description'],
            $mappingSettings['product_description'],
            $mappingSettings['short_product_description']

        ];
        if ($this->getCustomOpenGraph()) {
            $fieldsToSelect[] = $mappingSettings['open_graph'];
        }
        $collection->addFieldToSelect($fieldsToSelect);

        if ($productId) {
            $collection->addFieldToFilter('entity_id', ['neq' => $productId]);
        }

        if ($search) {
            $collection->addFieldToFilter('name', ['like' => '%' . $search . '%']);
        }

        $collection->addAttributeToSort('name', 'ASC');

        $total = $collection->getSize();

        $collection->getSelect()->limit($page * $pageSize);

        $result = [];
        foreach ($collection as $product) {
            $pageTitle = $this->getAttributesValues($product, 'page_title');
            $pageDescription = $this->getAttributesValues($product, 'page_description');
            $productDescription = $this->getAttributesValues($product, 'product_description');
            $shortProductDescription = $this->getAttributesValues($product, 'short_product_description');
            $productName = $this->catalogOutput->productAttribute(
                $product,
                $product->getName(),
                'name'
            );

            $data = [
                'value' => $product->getId(),
                'label' => $productName,
                'page_title' => $pageTitle,
                'page_description' => $pageDescription,
                'product_description' => $productDescription,
                'short_product_description' => $shortProductDescription
            ];

            if ($this->getCustomOpenGraph()) {
                $openGraph = $this->getAttributesValues($product, 'open_graph');
                $data['open_graph'] = $openGraph;
            }

            $result[] = $data;
        }

        return [
            'total' => $total,
            'items' => $result
        ];
    }

    /**
     * Get attributes values
     *
     * @param \Magento\Catalog\Api\Data\ProductInterface $product
     * @param string $field
     * @return string
     */
    public function getAttributesValues($product, $field)
    {
        $mapping = $this->getMappingSettings();
        $html = '';

        if (isset($mapping[$field])) {
            $attribute = $mapping[$field];
            $value = $product->getData($attribute);
            
            if ($value) {
                $html = $this->catalogOutput->productAttribute($product, $value, $attribute);
            }
        }

        return $html;
    }

    /**
     * Get Automatic Text Optimizations options
     *
     * @param bool $usedInFilter
     * @return array
     */
    public function getAutomaticTextOptimizationOptions($usedInFilter = false)
    {
        $automaticTextOptimization = $this->automaticTextOptimization->toOptionArray();
        if ($usedInFilter) {
            $automaticTextOptimization = $this->automaticTextOptimization->toOptionArrayForFilters();
        }
        return $automaticTextOptimization;
    }
    
    /**
     * Get force failed
     *
     * @return string
     */
    public function getForceFailed()
    {
        return $this->getDevSettings('force_failed');
    }

    /**
     * Get ko dev
     *
     * @return string
     */
    public function getKoDev()
    {
        return $this->getDevSettings('ko_dev');
    }

    /**
     * Get first visit cannibalization
     *
     * @return string
     */
    public function isFirstVisitCannibalization()
    {
        $visited = filter_var($this->getCannibalizationSettings('first_visit'), FILTER_VALIDATE_BOOLEAN);

        return !$visited;
    }
}
