<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class Tooltip extends TooltipCategory
{
    public const FAQ_URL = 'https://writetext.ai/frequently-asked-questions#credit-cost';

    public const KEYWORDS_GUIDE_URL = 'https://writetext.ai/guide-mastering-keyword-optimization';

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * Constructor
     *
     * @param SettingManager $settingManager
     * @param Data $helper
     * @param Settings $settings
     */
    public function __construct(
        SettingManager $settingManager,
        Data $helper,
        Settings $settings
    ) {
        $this->settingManager = $settingManager;
        $this->helper = $helper;
        $this->settings = $settings;
        parent::__construct($settingManager, $helper, $settings);
    }

    /**
     * Get rules
     *
     * @return array
     */
    public function getRules()
    {
        return $this->settings->getSettings('rules');
    }

    /**
     * Get tooltip
     *
     * @param string $block
     * @param string $entityType
     *
     * @return string
     */
    public function getTooltip($block, $entityType = null)
    {
        $tooltip = '';
        switch ($block) {
            case 'wtai.reviewed':
                $tooltip = $this->getReviewedTooltip();
                break;
            case 'wtai.keywords':
                $tooltip = $this->getKeywordsTooltip();
                break;
            case 'wtai.attributes':
                if ($entityType === 'Category') {
                    $tooltip = $this->getRepresentativesTooltip();
                } else {
                    $tooltip = $this->getAttributesTooltip();
                }
                break;
            case 'wtai.tones.and.styles':
                $tooltip = $this->getTonesAndStylesTooltip();
                break;
            case 'wtai.audience':
                $tooltip = $this->getAudienceTooltip();
                break;
            case 'wtai.page.title':
                $tooltip = $this->getPageTitleTooltip();
                break;
            case 'wtai.product.description':
                $tooltip = $this->getProductDescriptionTooltip();
                break;
            case 'wtai.short.description':
                $tooltip = $this->getShortDescriptionTooltip();
                break;
            case 'wtai.open.graph':
                $tooltip = $this->getOpenGraphTooltip();
                break;
            case 'wtai.page.description':
                $tooltip = $this->getPageDescriptionTooltip();
                break;
            case 'wtai.reference.product':
                $tooltip = $this->getReferenceProductTooltip();
                break;
            case 'wtai.generate.rewrite':
                $tooltip = $this->getGenerateRewriteTooltip();
                break;
            case 'wtai.transfer':
                $tooltip = $this->getTransferTooltip();
                break;
            case 'wtai.preview.changes':
                $tooltip = $this->getPreviewChangesTooltip();
                break;
            case 'wtai.alt.text':
                $tooltip = $this->getAltTextTooltip();
                break;
            case 'wtai.special.instructions':
                $tooltip = $this->getSpecialInstructionsTooltip();
                break;
            case 'wtai.trigger.action.none.selected':
                $tooltip = $this->getTriggerActionNoneSelectedTooltip();
                break;
            case 'wtai.intent':
                $tooltip = $this->getIntentTooltip();
                break;
            case 'wtai.pipeline.locked':
                $tooltip = $this->getPipelineLockedTooltip();
                break;
            case 'wtai.pipeline.unlocked':
                $tooltip = $this->getPipelineUnlockedTooltip();
                break;
            case 'wtai.add.keywords':
                $tooltip = $this->getAddKeywordsTooltip();
                break;
            case 'wtai.currently.optimizing.keywords':
                $tooltip = $this->getCurrentlyOptimizingKeywordsTooltip();
                break;
        }
        
        // If tooltip is empty, check tooltip from category
        if (empty($tooltip)) {
            $tooltip = $this->getTooltipCategory($block);
        }

        return $tooltip;
    }

    /**
     * Get add keywords tooltip
     *
     * @return string
     */
    public function getAddKeywordsTooltip()
    {
        return __("(Optional) Enter seed keyword/s that relate to your product."
            . " These will be used to kickstart the AI-powered keyword analysis process."
            . " Seed keywords are the foundation of effective optimization,"
            . " acting as the starting point for identifying high-potential keywords tailored to your content."
            . " If you don't enter any keywords here, WriteText.ai will automatically supply"
            . " seed keywords based on your product name and AI image analysis.");
    }

    /**
     * Get special instructions tooltip
     *
     * @return string
     */
    public function getSpecialInstructionsTooltip()
    {
        return __('Enter other details or instructions that you would like WriteText.ai'
            . ' to consider in generating text for the products you selected'
            . ' (e.g., add information on discounts, special sales, etc).'
            . ' This will not replace any details you may have added in the “Other product details”'
            . ' section in the individual product pages.'
            . ' Special instructions will be cleared at the end of a user session.');
    }

    /**
     * Get intent tooltip
     *
     * @return string
     */
    public function getIntentTooltip()
    {
        return sprintf('<strong>%s</strong>', __('Intent guide:'))
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Transactional'),
                __('Users aim to complete a transaction, such as making a purchase or signing up for a service.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Commercial'),
                __('Users are researching products or services with the intention of making a purchase decision soon.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Navigational'),
                __('Users have a specific website or web page in mind and are'
                    . ' using search engines to navigate there directly.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Informational'),
                __('Users seek information on a specific topic, intending to learn something.')
            );
    }

    /**
     * Get reviewed tooltip
     *
     * @return string
     */
    public function getReviewedTooltip()
    {
        return sprintf('<p>%s</p>', __('Check this box to keep track of the products'
                . ' where you have reviewed the text. This is especially helpful if you'
                . ' have an internal workflow where text needs to go through a review'
                . ' process first before being published on the website. This checkbox'
                . ' does not affect the live content, it is only a classification.'))
            . sprintf('<p>%s</p>', __('You can filter reviewed products by'
                . ' clicking on the filter icon in the product list,'
                . ' clicking the WriteText.ai status dropdown and selecting “Reviewed”.'));
    }

    /**
     * Get keywords drawer tooltip
     *
     * @return string
     */
    public function keywordsDrawerTooltip()
    {
        // phpcs:disable Generic.Files.LineLength
        return sprintf('<p><a href="%s" target="_blank">%s</a></p>', self::KEYWORDS_GUIDE_URL, __('Read the complete guide here'))
            . sprintf("<p>%s</p>", __("This pipeline visually maps out the journey of keywords through different stages, helping you track progress and plan your next steps."))
            . sprintf("<p>%s</p>", __("Arrows show the progression path for each keyword."))
            . sprintf("<p>%s</p>", __("As your product page meet the trigger criteria, it automatically moves up to the next keyword."))
            . sprintf("<p>%s</p>", __("You can set different triggers based on your SEO strategy:<br>- When your product page is ranking within a specific position range in the SERP (recommended); or<br>- After a certain number of days"))
            . sprintf("<p>%s</p>", __("Use the Ranking History section to track progress over time."))
            . sprintf("<p>%s</p>", __("Select a specific date or refresh the view to get the most up-to-date ranking information."))
            . sprintf("<p>%s</p>", __("Click on a specific keyword to view status, position in the SERP, overview of the SERP, difficulty, search volume, traffic potential, and intent."))
            . sprintf("<p>%s</p>", __("Cluster-based pipelines are automatically organized based on the best possible keywords for each difficulty stage according to their search volume. Other keyword ideas that were not selected for the cluster-based pipelines are in the keyword pool below. If you want to use these keywords, you can build a custom pipeline by dragging keywords from the pool into an empty pipeline. You can also manually add target keywords here and WriteText.ai will automatically retrieve search volume and difficulty data if available."));
        // phpcs:enable Generic.Files.LineLength
    }

    /**
     * Get keywords tooltip
     *
     * @return string
     */
    public function getKeywordsTooltip()
    {
        // phpcs:disable Generic.Files.LineLength
        return  sprintf('<p><a href="%s" target="_blank">%s</a></p>', self::KEYWORDS_GUIDE_URL, __('Read the complete guide here'))
            . sprintf("<p>%s</p>", __('The keyword optimization pipeline is an AI-powered keyword analysis tool that focuses on gradual, targeted keyword optimization. Click Start AI-powered keyword analysis to get keyword ideas and have them automatically organized into cluster-based pipelines that start with low-difficulty keywords and gradually move up to higher-difficulty and higher-volume keywords.'))
            . sprintf("<p>%s</p>", __('Optionally, you can add "seed" or starting keywords. Adding a keyword as a seed does not guarantee that they will be automatically included in the cluster-based pipelines. You have the option to manually add your target keywords or create custom pipelines if you want to set specific keywords to target or optimize for.'))
            . sprintf("<p>%s</p>", __('Due to the complex nature of this process, starting AI-powered keyword analysis may take some time. You can click on "Automatically generate/rewrite text when keyword analysis is done" to streamline the process. Make sure to select the text types you want to generate before starting AI-powered keyword analysis.'))
            . sprintf("<p><b>%s</b></p>", __('How the keyword optimization pipeline works'))
            . sprintf("<p>%s</p>", __('Starting with low-difficulty keywords:<br>The pipeline identifies keywords with lower competition but good search potential.'))
            . sprintf("<p>%s</p>", __('Progressive optimization:<br>As your product page starts to rank for low-difficulty keywords, the pipeline gradually recommends optimizing for higher-difficulty keywords.'))
            . sprintf("<p>%s</p>", __('Automation and notifications:<br>WriteText.ai automates much of this process, analyzing keyword trends and notifying you when pages are ready for the next optimization step. Depending on your preferences, it can automatically rewrite content or prompt you to check and initiate the rewriting yourself.'))
            . sprintf("<p>%s</p>", __('Iterative content refinement:<br>At every stage, the pipeline ensures your content is updated and optimized for current keyword trends, helping you maintain relevance and visibility.'));
        // phpcs:enable Generic.Files.LineLength
    }

    /**
     * Get attributes tooltip
     *
     * @return string
     */
    public function getAttributesTooltip()
    {
        return  sprintf("<p>%s</p>", __("Check the product attributes that you want to be"
                . " considered in generating text. You can also select your main product image here"
                . " and AI will analyze it in order to generate more accurate and relevant text."
                . " Please make sure that the image accurately represents the product (i.e., it is"
                . " not some kind of placeholder or a generic image for your shop)."))
            . sprintf("<p>%s</p>", __('There is no guarantee that a selected product attribute'
                . ' will always appear in the text, but it will help guide WriteText.ai'
                . ' in generating more relevant text.'))
            . sprintf("<p>%s</p>", __('If you want to add more information about the product,'
                . ' you can enter them in the “Other product details” section.'
                . ' For example, you can write:'))
            . sprintf("<p><em>%s</em>", __("“The shirt comes in a reusable shirt bag that can be used for traveling.”"))
            . sprintf("<br/><br/>%s", __(
                "WriteText.ai will then attempt to include the text or its meaning in the generated product text, e.g.:"
            ))
            . sprintf("<br/><br/><em>%s</em></p>", __(
                "“...Plus, its reusable bag makes it perfect for travel or to keep in storage when not in use!”"
            ));
    }

    /**
     * Get tones and styles tooltip
     *
     * @return string
     */
    public function getTonesAndStylesTooltip()
    {
        return __('<p>Select one or more tones to set the overall mood'
            . ' and attitude of the text, and then choose a style to'
            . ' set the voice and structure of the text.</p>');
    }

    /**
     * Get audience tooltip
     *
     * @return string
     */
    public function getAudienceTooltip()
    {
        return __('<p>Select the applicable audience for your product so WriteText.ai'
            . ' can generate text that will appeal to them.  If you don’t select an audience,'
            . ' the generated text will default to a “neutral” audience. WriteText.ai also'
            . ' suggests more specific target markets based on your keywords. Click on the'
            . ' audience to select it or type in your own custom audience in the box.</p>');
    }

    /**
     * Get meta title tooltip
     *
     * @return string
     */
    public function getPageTitleTooltip()
    {
        $pageTitleLength = $this->helper->getMaxLengthSettings('page_title');

        return sprintf(__('<p>WriteText.ai aims to generate a title with around'
            . ' %s characters, based on current SEO best practices. The current'
            . ' meta title saved for the product is displayed in the box on'
            . ' the right for your reference only.</p>'), $pageTitleLength);
    }

    /**
     * Get meta description tooltip
     *
     * @return string
     */
    public function getPageDescriptionTooltip()
    {
        $maxPageDescriptionLength = $this->helper->getMaxLengthSettings('page_description');

        return sprintf(__('<p>WriteText.ai aims to generate a description with'
            . ' around %s characters, based on current SEO best practices. The current'
            . ' meta description saved for the product is displayed in the box on'
            . ' the right for your reference only.</p>'), $maxPageDescriptionLength);
    }

    /**
     * Get description tooltip
     *
     * @return string
     */
    public function getProductDescriptionTooltip()
    {
        return __('<p>Please check your website to see if you are using the description or'
                . ' the short description field (or both) to ensure that you are generating the'
                . ' correct text. The current description saved for the product is displayed'
                . ' in the box on the right for your reference only.</p>')
            . __('<p>Target length<br/>')
            . __('Indicate your target length by setting a minimum and maximum word count.'
                . ' WriteText.ai will aim to generate text within the number you set,'
                . ' but it may give you more or less words than expected from time to time.'
                . ' Note that your credit cost will depend on the target length you'
                . ' set so make sure to set a reasonable target range.');
    }

    /**
     * Get short description tooltip
     *
     * @return string
     */
    public function getShortDescriptionTooltip()
    {
        return sprintf('<p>%s</p>', __('Please check your website to see if'
                . ' you are using the description or the short description field (or both)'
                . ' to ensure that you are generating the correct text. The current short'
                . ' description saved for the product is displayed in the box on'
                . ' the right for your reference only.'))
            . __('<p>Target length<br/>')
            . __('Indicate your target length by setting a minimum and maximum word count.'
                . ' WriteText.ai will aim to generate text within the number you set,'
                . ' but it may give you more or less words than expected from time to time.'
                . ' Note that your credit cost will depend on the target length you'
                . ' set so make sure to set a reasonable target range.');
    }

    /**
     * Get open graph tooltip
     *
     * @return string
     */
    public function getOpenGraphTooltip()
    {
        $maxOpenGraphLength = $this->helper->getMaxLengthSettings('open_graph');

        return sprintf(__('<p>This is the text that appears in the preview when you share'
            . ' the product page on social media sites like Facebook, Twitter, or LinkedIn.'
            . ' WriteText.ai aims to generate text with more or less %s characters,'
            . ' based on current best practices. The current Open Graph text saved for'
            . ' the product is displayed in the box on the right'
            . ' for your reference only.</p>'), $maxOpenGraphLength);
    }

    /**
     * Get transfer tooltip
     *
     * @return string
     */
    public function getTransferTooltip()
    {
        return sprintf("<p>%s</p>", __("When you're done generating and/or editing text,"
                . " you have the option to save the draft inside WriteText.ai or transfer"
                . " the text to Magento. Transferring your text to Magento will publish it"
                . " directly to the website. If the updates did not reflect in your page,"
                . " you may need to flush the cache or run reindexing. Please also make"
                . " sure that your product is enabled in the frontend."))
            . sprintf("<p>%s</p>", __("Note: Any media or directive you have inserted in"
                . " your current Magento text will be overwritten when you transfer from WriteText.ai."));
    }

    /**
     * Get reference product tooltip
     *
     * @return string
     */
    public function getReferenceProductTooltip()
    {
        return sprintf('<p>%s</p>', __("If you have already edited a certain product’s texts"
                . " to fit the specific style and structure that you want, you can set it as a"
                . " “Reference product” which WriteText.ai will use as the basis for generating text."))
            . sprintf('<p>%s</p>', __("When using a reference product,"
                . " the individual settings for tone, style, and audience will be ignored,"
                . " unless the specific text type you’re generating does not have a counterpart"
                . " in your reference product, in which case WriteText.ai will revert to"
                . " the individual settings for tone, style, and audience."))
            . sprintf(
                '<p>%s</p>',
                __("Note that image alt text generation will not be affected by the reference product.")
            )
            . sprintf(__('The credit cost for generating text using a reference product will'
                . ' depend on the length of the reference text and any formatting'
                . ' (e.g., bold, italics, numbered or bulleted lists) applied to it.'
                . ' For a more detailed explanation, you may read our FAQ section on'
                . ' <a href="%s" target="_blank">how credit cost is calculated</a>.'), self::FAQ_URL);
    }

    /**
     * Get generate rewrite tooltip
     *
     * @return string
     */
    public function getGenerateRewriteTooltip()
    {
        return sprintf('<p class="wtai-heading">%s</p>', __('Generate new text'))
            . sprintf('<p>%s</p>', __('Generate text for your selected text types based on the settings on this page.'))
            . sprintf('<p class="wtai-heading">%s</p>', __('Rewrite existing'))
            . sprintf('<p>%s</p>', __('If any of your settings'
                . ' (e.g., keywords, product attributes, tone, style, and audiences)'
                . ' have changed but you don’t necessarily want to generate completely new text,'
                . ' you can use the Rewrite feature to keep the existing structure of the text but'
                . ' rewrite it to reflect any changes. Note that for the image alt text,'
                . ' only the product name and keywords are considered in the rewrite.'))
            . sprintf('<p>%s</p>', __('Note: Any media or directive you have inserted into the existing text'
                . ' will not be included in the rewritten version. The language of the text you'
                . ' are rewriting will be the same language used in the output.'))
            . sprintf(__('The credit cost for rewriting text will depend'
                . ' on the length of the original text and any formatting'
                . ' (e.g., bold, italics, numbered or bulleted lists) applied to it.'
                . ' For a more detailed explanation, you may read our FAQ section on <a href="%s" target="_blank">'
                . 'how your credit cost is calculated</a>.'), self::FAQ_URL);
    }

    /**
     * Get preview changes tooltip
     *
     * @return string
     */
    public function getPreviewChangesTooltip()
    {
        return __('Save your changes to preview how they will look on your website.');
    }

    /**
     * Get alt text tooltip
     *
     * @return string
     */
    public function getAltTextTooltip()
    {
        $altTextLength = $this->helper->getMaxLengthSettings('alt_text');

        return sprintf(__("Analyze your product image and automatically generate an alt image text"
            . " (incorporating your product name and keywords) to help improve your website's"
            . " accessibility and the way that search engines understand your content."
            . " WriteText.ai aims to generate an alt text with around %s characters based on current best practices."
            . " Note: We only allow up to 10 images here, including the product featured image."
            . " If you have more than 10 images uploaded to your product,"
            . " only the first 10 images in your listing will be included."), $altTextLength);
    }

    /**
     * Get trigger action 'none' selected tooltip
     *
     * @return string
     */
    public function getTriggerActionNoneSelectedTooltip()
    {
        return __('This option is disabled when trigger action selected is "None".');
    }

    /**
     * Get trigger action 'none' selected tooltip
     *
     * @return string
     */
    public function getPipelineLockedTooltip()
    {
        return __("This pipeline is currently unlocked. Click to lock the entire pipeline to"
            . " preserve its current structure  and prevent changes, regardless of monthly search data updates.");
    }

    /**
     * Get trigger action 'none' selected tooltip
     *
     * @return string
     */
    public function getPipelineUnlockedTooltip()
    {
        return __("This pipeline is currently locked. Click to unlock the keyword pipeline to"
            . " allow updates and adjustments based on the latest search data trends.");
    }

    /**
     * Get currently optimizing keywords tooltip
     *
     * @return string
     */
    public function getCurrentlyOptimizingKeywordsTooltip()
    {
        return __('These are the keywords currently applied in the text,'
            . ' either used exactly as is or integrated in a semantically relevant way.');
    }
}
