<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class TooltipCategory implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    public const FAQ_URL = 'https://writetext.ai/frequently-asked-questions#credit-cost';

    public const KEYWORDS_GUIDE_URL = 'https://writetext.ai/guide-mastering-keyword-optimization';

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * Constructor
     *
     * @param SettingManager $settingManager
     * @param Data $helper
     * @param Settings $settings
     */
    public function __construct(
        SettingManager $settingManager,
        Data $helper,
        Settings $settings
    ) {
        $this->settingManager = $settingManager;
        $this->helper = $helper;
        $this->settings = $settings;
    }

    /**
     * Get tooltip
     *
     * @param string $block
     * @return string
     */
    public function getTooltipCategory($block)
    {
        $tooltip = '';
        switch ($block) {
            case 'wtai.reviewed.category':
                $tooltip = $this->getReviewedTooltipCategory();
                break;
            case 'wtai.keywords.category':
                $tooltip = $this->getKeywordsTooltipCategory();
                break;
            case 'wtai.generate.rewrite.category':
                $tooltip = $this->getGenerateRewriteTooltipCategory();
                break;
            case 'wtai.audience.category':
                $tooltip = $this->getAudienceTooltipCategory();
                break;
            case 'wtai.category.description':
                $tooltip = $this->getCategoryDescriptionTooltip();
                break;
            case 'wtai.category.image':
                $tooltip = $this->getCategoryImageTooltip();
                break;
            case 'wtai.page.title.category':
                $tooltip = $this->getPageTitleTooltipCategory();
                break;
            case 'wtai.page.description.category':
                $tooltip = $this->getPageDescriptionTooltipCategory();
                break;
            case 'wtai.transfer.category':
                $tooltip = $this->getTransferTooltipCategory();
                break;
        }

        return $tooltip;
    }

    /**
     * Get representative products tooltip
     *
     * @return string
     */
    public function getRepresentativesTooltip()
    {
        $rules = $this->getRules();
        $maxRepresentativeProducts = $rules['maxRepresentativeProducts'] ?? '';

        return sprintf(
            __('Select a maximum of %s products that best represent this category.'
                . ' Depending on the data available, the product name, product description, and'
                . ' product images will be used as reference for generating text for this category.'),
            $maxRepresentativeProducts
        );
    }

    /**
     * Get meta title tooltip for category
     *
     * @return string
     */
    public function getPageTitleTooltipCategory()
    {
        $pageTitleLength = $this->helper->getMaxLengthSettings('page_title');

        return sprintf(__('<p>WriteText.ai aims to generate a title with around'
            . ' %s characters, based on current SEO best practices. The current'
            . ' meta title saved for the category is displayed in the box on'
            . ' the right for your reference only.</p>'), $pageTitleLength);
    }

    /**
     * Get meta description tooltip for category
     *
     * @return string
     */
    public function getPageDescriptionTooltipCategory()
    {
        $maxPageDescriptionLength = $this->helper->getMaxLengthSettings('page_description');

        return sprintf(__('<p>WriteText.ai aims to generate a description with'
            . ' around %s characters, based on current SEO best practices. The current'
            . ' meta description saved for the category is displayed in the box on'
            . ' the right for your reference only.</p>'), $maxPageDescriptionLength);
    }

    /**
     * Get generate rewrite tooltip category
     *
     * @return string
     */
    public function getGenerateRewriteTooltipCategory()
    {
        return sprintf('<p class="wtai-heading">%s</p>', __('Generate new text'))
            . sprintf('<p>%s</p>', __('Generate text for your selected text types based on the settings on this page.'))
            . sprintf('<p class="wtai-heading">%s</p>', __('Rewrite existing'))
            . sprintf('<p>%s</p>', __('If any of your settings'
                . ' (e.g., representative products, keywords, tone, style, and audiences)'
                . ' have changed but you don’t necessarily want to generate completely new text,'
                . ' you can use the Rewrite feature to keep the existing structure of the text but'
                . ' rewrite it to reflect any changes.'))
            . sprintf('<p>%s</p>', __('Note: Any media or directive you have inserted into the existing text'
                . ' will not be included in the rewritten version. The language of the text you'
                . ' are rewriting will be the same language used in the output.'))
            . sprintf(__('The credit cost for rewriting text will depend'
                . ' on the length of the original text and any formatting'
                . ' (e.g., bold, italics, numbered or bulleted lists) applied to it.'
                . ' For a more detailed explanation, you may read our FAQ section on <a href="%s" target="_blank">'
                . 'how your credit cost is calculated</a>.'), self::FAQ_URL);
    }

    /**
     * Get transfer tooltip for Category
     *
     * @return string
     */
    public function getTransferTooltipCategory()
    {
        return sprintf("<p>%s</p>", __("When you're done generating and/or editing text,"
                . " you have the option to save the draft inside WriteText.ai or transfer"
                . " the text to Magento. Transferring your text to Magento will publish it"
                . " on the website."))
            . sprintf("<p>%s</p>", __("Note: Any media or directive you have inserted in"
                . " your current Magento text will be overwritten when you transfer from WriteText.ai."));
    }

    /**
     * Get reviewed tooltip for category
     *
     * @return string
     */
    public function getReviewedTooltipCategory()
    {
        return sprintf('<p>%s</p>', __('Check this box to keep track of the categories'
                . ' where you have reviewed the text. This is especially helpful if you'
                . ' have an internal workflow where text needs to go through a review'
                . ' process first before being published on the website. This checkbox'
                . ' does not affect the live content, it is only a classification.'))
            . sprintf('<p>%s</p>', __('You can filter reviewed categories by'
                . ' selecting “Reviewed” under the “Filter by WriteText.ai status”'
                . ' dropdown in the category list.'));
    }

    /**
     * Get audience tooltip for category
     *
     * @return string
     */
    public function getAudienceTooltipCategory()
    {
        return __('<p>Select the applicable audience for your category so WriteText.ai'
            . ' can generate text that will appeal to them.  If you don’t select an audience,'
            . ' the generated text will default to a “neutral” audience. WriteText.ai also'
            . ' suggests more specific target markets based on your keywords. Click on the'
            . ' audience to select it or type in your own custom audience in the box.</p>');
    }

    /**
     * Get keywords tooltip for category
     *
     * @return string
     */
    public function getKeywordsTooltipCategory()
    {
        // phpcs:disable Generic.Files.LineLength
        return  sprintf('<p><a href="%s" target="_blank">%s</a></p>', self::KEYWORDS_GUIDE_URL, __('Read the complete guide here'))
            . sprintf("<p>%s</p>", __('The keyword optimization pipeline is an AI-powered keyword analysis tool that focuses on gradual, targeted keyword optimization. Click Start AI-powered keyword analysis to get keyword ideas and have them automatically organized into cluster-based pipelines that start with low-difficulty keywords and gradually move up to higher-difficulty and higher-volume keywords.'))
            . sprintf("<p>%s</p>", __('Optionally, you can add "seed" or starting keywords. Adding a keyword as a seed does not guarantee that they will be automatically included in the cluster-based pipelines. You have the option to manually add your target keywords or create custom pipelines if you want to set specific keywords to target or optimize for.'))
            . sprintf("<p>%s</p>", __('Due to the complex nature of this process, starting AI-powered keyword analysis may take some time. You can click on "Automatically generate/rewrite text when keyword analysis is done" to streamline the process. Make sure to select the text types you want to generate before starting AI-powered keyword analysis.'))
            . sprintf("<p><b>%s</b></p>", __('How the keyword optimization pipeline works'))
            . sprintf("<p>%s</p>", __('Starting with low-difficulty keywords:<br>The pipeline identifies keywords with lower competition but good search potential.'))
            . sprintf("<p>%s</p>", __('Progressive optimization:<br>As your product page starts to rank for low-difficulty keywords, the pipeline gradually recommends optimizing for higher-difficulty keywords.'))
            . sprintf("<p>%s</p>", __('Automation and notifications:<br>WriteText.ai automates much of this process, analyzing keyword trends and notifying you when pages are ready for the next optimization step. Depending on your preferences, it can automatically rewrite content or prompt you to check and initiate the rewriting yourself.'))
            . sprintf("<p>%s</p>", __('Iterative content refinement:<br>At every stage, the pipeline ensures your content is updated and optimized for current keyword trends, helping you maintain relevance and visibility.'));
        // phpcs:enable Generic.Files.LineLength
    }

    /**
     * Get category description tooltip
     *
     * @return string
     */
    public function getCategoryDescriptionTooltip()
    {
        return __('<p>The current category description saved for the category is displayed'
                . ' in the box on the right for your reference only.')
            . __('<p>Target length<br/>')
            . __('<p>Indicate your target length by setting a minimum and maximum word count.'
                . ' WriteText.ai will aim to generate text within the number you set,'
                . ' but it may give you more or less words than expected from time to time.'
                . ' Note that your credit cost will depend on the target length you'
                . ' set so make sure to set a reasonable target range.</p>');
    }

    /**
     * Get category image tooltip
     *
     * @return string
     */
    public function getCategoryImageTooltip()
    {
        return __('<p>If available, you can let AI analyze the category image in order'
                .' to generate more accurate and relevant text. Please check that the image'
                .' accurately represents the category (i.e., it is not some kind of'
                .' placeholder or a generic image for your shop).</p>');
    }

    /**
     * Get intent tooltip
     *
     * @return string
     */
    public function getCategoryIntentTooltip()
    {
        return sprintf('<strong>%s</strong>', __('Intent guide:'))
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Transactional'),
                __('Users aim to complete a transaction, such as making a purchase or signing up for a service.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Commercial'),
                __('Users are researching products or services with the intention of making a purchase decision soon.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Navigational'),
                __('Users have a specific website or web page in mind and are'
                    . ' using search engines to navigate there directly.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Informational'),
                __('Users seek information on a specific topic, intending to learn something.')
            );
    }
}
