/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields'
], function (
    $,
    ko,
    Component,
    prompt,
    textfieldsUtils
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            tooltipSelector: '.wtai-bulk-generate-form .admin__field-tooltip-action',
            clusterCount: 4,
            isLoading: true,
            searchIntent: [],
            searchIntents: [
                'transactional',
                'commercial',
                'navigational',
                'informational'
            ],
            autoTextOptimizationSelected: 'flag'
        },
        
        isSpinning: false,
        isReopening: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            $('.wtai-bulk-keyword-analysis-form-container > *').applyBindings(this);

            self = this;

            self.autoTextOptimizationSelected.subscribe(function (autoTextOptimizationSelected) {
                prompt.autoTextOptimizationSelected(autoTextOptimizationSelected);
                self.savePreferences('automatic_text_optimization', autoTextOptimizationSelected);
            });

            self.searchIntent.subscribe(function (searchIntent) {
                prompt.searchIntent(searchIntent);
                if (!self.isReopening) {
                    self.savePreferences('search_intent', searchIntent.join(','));
                }
            });

            self.clusterCount.subscribe(function (clusterCount) {
                prompt.clusterCount(clusterCount);
                if (!self.isSpinning) {
                    self.savePreferences('cluster_count', clusterCount);
                }
            });
            
            this.initTooltip();
            this.initSpinner();
            this.initPreferences();
            this.removeLoader();
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'isLoading',
                    'searchIntent',
                    'searchIntents',
                    'clusterCount',
                    'autoTextOptimizationSelected'
                ]);

            this.searchIntentCommadSeparated = ko.computed(function () {
                return this.searchIntent().join(', ');
            }, this);

            return this;
        },

        /**
         * Initialize preferences
         *
         * Set model values on page load.
         * Model values updates on change.
         * Retain selected fields when reopening the modal.
         */
        initPreferences: function () {
            if (!prompt.keywordAnalysisPromptInitialized()) {
                prompt.autoTextOptimizationSelected(this.autoTextOptimizationSelected());
                prompt.searchIntent(this.searchIntent());
                prompt.clusterCount(this.clusterCount());
                prompt.keywordAnalysisPromptInitialized(true);
            } else {
                this.isReopening = true;
                this.autoTextOptimizationSelected(prompt.autoTextOptimizationSelected());
                this.searchIntent(prompt.searchIntent());
                this.clusterCount(prompt.clusterCount());
                this.isReopening = false;
            }
        },

        /**
         * Save user preferences
         *
         * @param {String} field
         * @param {String} selected
         *
         * @return {void}
         */
        savePreferences: function (field, selected) {
            var value = selected;
            
            if (value.length <= 0) {
                value = '';
            }

            $.ajax({
                url: self.saveUrl,
                type: 'POST',
                data: {
                    [field]: value,
                    entity_type: 'product',
                    scope: 'grid'
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Restore user preferences
         */
        restore: function () {
            this.autoTextOptimizationSelected(this.globalSettings.automatic_text_optimization);
            this.searchIntent([...this.globalSettings.search_intent]);
            this.clusterCount(this.globalSettings.cluster_count);

            $.ajax({
                url: this.restoreUrl,
                type: 'GET',
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Initialize spinner
         */
        initSpinner: function () {
            setTimeout(
                function () {
                    $('.wtai-bulk-keyword-analysis-popup .wtai-cluster-count').spinner({
                        numberFormat: 'n',
                        min: this.minClusterCount,
                        max: this.maxClusterCount,
                        spin: function (event, ui) {
                            self.isSpinning = true;
                            self.clusterCount(ui.value);
                            self.isSpinning = false;
                        },
                        change: function (event) {
                            self.clusterCount(event.target.value);
                        }
                    });
                }.bind(this),
                100
            );
        },

        /**
         * Initialize tooltip
         */
        initTooltip: function () {
            $(self.tooltipSelector).dropdown({
                parent: '.admin__field-tooltip',
                autoclose: true
            });
        },

        /**
         * Remove loader
         */
        removeLoader: function () {
            setTimeout(function () {
                self.isLoading(false);
            }, 100);
        },

        /**
         * Is intent selected
         */
        isIntentSelected: function (intent) {
            return this.searchIntent().indexOf(intent) !== -1;
        },

        /**
         * Select search intent
         *
         * @param {String} intent
         * @return {void}
         */
        selectIntent: function (intent) {
            var searchIntent = this.searchIntent();

            if (searchIntent.length <= 1 &&
                searchIntent.indexOf(intent) !== -1
            ) {
                return;
            }

            if (searchIntent.indexOf(intent) === -1) {
                searchIntent.push(intent);
            } else {
                searchIntent = _.without(searchIntent, intent);
            }

            this.searchIntent(searchIntent);
        },

        /**
         * Validate min and max fields.
         *
         * @param {Object} data
         * @param {Object} event
         *
         * @return {void}
         */
        validateMinMax: function (data, event) {
            var target = event.target,
                value = Number(target.value),
                min = Number(target.min),
                max = Number(target.max),
                name = target.name;

            const mapping = {
                cluster_count: self.clusterCount
            };

            if (value < min || value > max) {
                const func = mapping[name],
                    val = value < min ? min : max;

                $(target).val(val);
                func(val);
            }
        },

        /**
         * Validate number.
         *
         * @param {Object} data
         * @param {Object} event
         *
         * @return {Boolean}
         */
        ensureNumerical: function (data, event) {
            return textfieldsUtils.ensureNumerical(event.key);
        },
    });
});
