/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'underscore',
    'Magento_Ui/js/grid/massactions',
    'WriteTextAI_WriteTextAI/js/modal/confirm',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/listing',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'mage/translate',
    'WriteTextAI_WriteTextAI/js/grid/reload',
    'WriteTextAI_WriteTextAI/js/utils/grid/actions',
    'WriteTextAI_WriteTextAI/js/model/grid/invalid-image-popup',
    'WriteTextAI_WriteTextAI/js/utils/grid/transfer',
    'WriteTextAI_WriteTextAI/js/utils/grid/generate',
    'WriteTextAI_WriteTextAI/js/utils/grid/keyword-analysis',
    'text!WriteTextAI_WriteTextAI/template/modal/bulk-generate.html',
    'text!WriteTextAI_WriteTextAI/template/modal/bulk-keyword-analysis.html',
    'WriteTextAI_WriteTextAI/js/utils/signalr',
    'text!ui/template/modal/modal-popup.html',
    'mage/validation'
], function (
    $,
    ko,
    _,
    Massactions,
    confirm,
    notifications,
    errorMessagesModel,
    listingModel,
    totalCreditsModel,
    $t,
    reloadGrid,
    gridUtils,
    gridInvalidImagePopupModel,
    transferUtils,
    generateUtils,
    keywordAnalysisUtils,
    bulkGeneratePopupTmpl,
    bulkKeywordAnalysisPopupTmpl,
    signalrUtils,
    modalPopupTmpl
) {
    'use strict';

    return Massactions.extend({
        defaults: {
            template: 'WriteTextAI_WriteTextAI/grid/actions',
            generateFormSelector: '#wtaiBulkGenerateForm',
            transferFormSelector: '#wtaiBulkTransferForm',
            keywordAnalysisFormSelector: '#wtaiBulkKeywordAnalysisForm',
            statusRunning: 'Running',
            defaultStoreId: 0,
            currentUser: {},
            isPremium: true,
            totalCredits: 0
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            this.isPremium.subscribe(function (isPremium) {
                if (typeof isPremium !== 'undefined') {
                    totalCreditsModel.isPremium(isPremium);
                }
            });

            this.currentUser.subscribe(function (currentUser) {
                if (currentUser && Object.keys(currentUser).length !== 0) {
                    listingModel.currentUser(currentUser);
                    gridInvalidImagePopupModel.currentUser(currentUser);
                    transferUtils.continueBulkTransfer(this);
                }
            }, this);

            this.totalCredits.subscribe(function (totalCredits) {
                totalCreditsModel.totalCredits(totalCredits);
            }, this);

            this.ids.subscribe(function (ids) {
                listingModel.ids(ids);
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'currentUser',
                    'isPremium',
                    'totalCredits',
                    'ids'
                ]);

            this.isDisabled = ko.computed(function () {
                if (!this.isPremium()) {
                    return true;
                }

                var bulkGenerateOngoing = notifications.statuses().some((status) => {
                    var statuses = ['Cancelled', 'Completed', 'Failed'];
                    if (signalrUtils.isCancelling(status) && status.user === this.currentUser().email) {
                        return true;
                    }
                    return (
                        !statuses.includes(status.status) &&
                        status.user === this.currentUser().email
                    );
                }, this);

                return bulkGenerateOngoing;
            }, this);

            return this;
        },

        /** @inheritdoc */
        applyAction: function (actionIndex) {
            if (this.isDisabled()) {
                return;
            }

            return this._super(actionIndex);
        },

        /**
         * Default action callback. Sends selections data
         * via POST request.
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         *
         * @returns {void}
         */
        defaultCallback: function (action, _data) {
            var settings = {};
            var data = JSON.parse(JSON.stringify(_data));

            errorMessagesModel.messages([]);

            try {
                switch (action.type) {
                    case 'generate':
                        settings = gridUtils.convertForm($(this.generateFormSelector));
                        generateUtils.bulkGenerate(this, action, data, settings);
                        break;
                    case 'transfer':
                        settings = gridUtils.convertForm($(this.transferFormSelector));
                        transferUtils.transferable(this, action, data, settings);
                        break;
                    case 'keyword_analysis':
                        settings = gridUtils.convertForm($(this.keywordAnalysisFormSelector));
                        keywordAnalysisUtils.bulkKeywordAnalysis(this, action, data, settings);
                        break;
                    default:
                        this._super(action, data);
                }
            } catch (e) {
                errorMessagesModel.messages.push(window.wtaGeneralErrorMessage);
                console.log('bulk ' + action.type + ' failed', e);
            }
        },

        /**
         * Bulk generate
         *
         * @param {Object} self - Self data.
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        bulkGenerateRequest: function (self, action, data, settings) {
            generateUtils.bulkGenerateRequest(self, action, data, settings);
        },

        /**
         * Bulk keyword analysis
         *
         * @param {Object} self - Self data.
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        bulkKeywordAnalysisRequest: function (self, action, data, settings) {
            keywordAnalysisUtils.bulkKeywordAnalysisRequest(self, action, data, settings);
        },

        /**
         * Reload grid.
         */
        gridReload: function () {
            var showLoader = false;

            reloadGrid.reloadUIComponent(
                'wtai_products_grid_listing.wtai_products_grid_listing_data_source',
                showLoader
            );
        },

        /**
         * Shows actions' confirmation window.
         *
         * @param {Object} action - Actions' data.
         * @param {Function} callback - Callback that will be
         *      invoked if action is confirmed.
         *
         * @returns {void}
         */
        _confirm: function (action, callback) {
            const actionType = action.type;
            const confirmData = action.confirm;

            if (!['generate', 'transfer', 'keyword_analysis'].includes(actionType)) {
                return this._super(action, callback);
            }

            const actionConfig = {
                generate: {
                    buttonText: confirmData.button,
                    modalClass: 'wtai-modal wtai-bulk-generate-popup wtai-confirm',
                    formSelector: this.generateFormSelector,
                    popupTpl: bulkGeneratePopupTmpl
                },
                transfer: {
                    buttonText: $t('Transfer'),
                    modalClass: 'wtai-modal wtai-bulk-transfer-popup wtai-transfer',
                    formSelector: this.transferFormSelector,
                    popupTpl: modalPopupTmpl
                },
                keyword_analysis: {
                    buttonText: $t('Start AI-powered keyword analysis'),
                    modalClass: 'wtai-modal wtai-bulk-keyword-analysis-popup',
                    formSelector: this.keywordAnalysisFormSelector,
                    popupTpl: bulkKeywordAnalysisPopupTmpl
                }
            };

            const config = actionConfig[actionType];

            confirm({
                creditCostText: $t('How is credit cost calculated?'),
                title: confirmData.title,
                content: confirmData.message,
                modalClass: config.modalClass,
                popupTpl: config.popupTpl,
                actions: {
                    confirm: callback,
                    cancel: () => $('body').removeClass('_has-modal')
                },
                buttons: [{
                    text: $t('Cancel'),
                    class: 'action-secondary action-dismiss',
                    click: function (event) {
                        this.closeModal(event);
                    }
                }, {
                    text: config.buttonText,
                    class: 'action-primary action-accept',
                    click: function (event) {
                        const $form = $(config.formSelector);
                        if ($form.data('validator') &&
                            $form.validation() &&
                            $form.validation('isValid')
                        ) {
                            this.closeModal(event, true);
                            $('body').removeClass('_has-modal');
                        }
                    }
                }]
            });
        }
    });
});
