/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'ko',
    'underscore',
    'mage/translate',
    'domReady!'
], function (
    ko,
    _,
    $t
) {
    'use strict';

    /**
     * @var {Array} statuses
     * @var {String} tempId
     * @var {String} transferTempId
     * @var {String} koTempId
     */
    var statuses = ko.observableArray([]),
        tempId = 'temp',
        transferTempId = 'transferTemp',
        koTempId = 'koTemp',
        opened = ko.observable(false);
        
    /**
     * Check if there is an intersection between the arrays
     *
     * @param {Array} array1
     * @param {Array} array2
     * @param {Array} array3
     *
     * @returns {Boolean}
     */
    function checkArraysIntersection(array1, array2 = [], array3)
    {
        for (let i = 0; i < array1.length; i++) {
            if (array3.includes(array1[i])) {
                return true;
            }
        }

        for (let i = 0; i < array2.length; i++) {
            if (array3.includes(array2[i])) {
                return true;
            }
        }

        return false;
    }

    /**
     * Get the status of the request
     *
     * @param {Object} messageEntry
     * @param {Object} remaining
     *
     * @returns {String}
     */
    function getMessage(messageEntry, remaining)
    {
        var message = '',
            text = '';

        switch (messageEntry.subType) {
            case 0:
            case 1:
                if (messageEntry.failed === messageEntry.total && messageEntry.failed > 0) {
                    return $t('Text generation failed, you can restart to try again.');
                }
                switch (messageEntry.status) {
                    case 'Pending':
                        message = $t('Generating text...');
                        break;
                    case 'Running':
                        if (remaining.remaining === 0) {
                            message = $t('Generating text...');
                            break;
                        }
                        
                        if (remaining.is_minutes) {
                            text = $t('Generating text... (Estimated time remaining %s minute/s)');
                        } else {
                            text = $t('Generating text... (Estimated time remaining %s second/s)');
                        }
                        message = text.replace('%s', remaining.remaining);
                        break;
                    case 'Completed':
                        message = $t('WriteText.ai is done generating text for your selected products.');
                        break;
                    case 'Cancelling':
                        message = $t('Some products have already been queued up for processing. Cancelling unqueued items only…');
                        break;
                    case 'Cancelled':
                        message = $t('This process has been cancelled.');
                        break;
                    case 'TimedOut':
                        message = $t('WriteText.ai is currently experiencing a high volume of requests and has timed out.');
                        break;
                    default:
                        message = $t('Generating text...');
                        break;
                }
                break;
            case 2:
                if (messageEntry.failed === messageEntry.total && messageEntry.failed > 0) {
                    return $t('Keyword analysis processing failed.');
                }
                switch (messageEntry.status) {
                    case 'Completed':
                        message = $t('Keyword analysis completed');
                        break;
                    case 'Cancelling':
                        message = $t('Some products have already been queued up for processing. Cancelling unqueued items only…');
                        break;
                    case 'Cancelled':
                        $t('Keyword analysis cancelled');
                        break;
                    case 'TimedOut':
                        message = $t('Keyword analysis timed out');
                        break;
                    case 'Running':
                        message = $t('Keyword analysis currently in progress');
                        break;
                    default:
                        message = $t('Keyword analysis currently in progress');
                        break;
                }
                break;
            default:
                break;
        }

        return message;
    }

    /**
     * Calculate the remaining time
     *
     * @param {String} estimatedEndTime
     * @returns {Object}
     */
    function calculateRemainingTime(estimatedEndTime)
    {
        var remaining = 0,
            isMinutes = false,
            result = {},
            currentDatetime,
            interval;

        if (estimatedEndTime !== null) {
            estimatedEndTime = new Date(estimatedEndTime);
            currentDatetime = new Date();
            interval = estimatedEndTime.getTime() - currentDatetime.getTime();
            remaining = Math.ceil(interval / 1000); /* Convert milliseconds to seconds */

            if (remaining >= 60) {
                remaining = Math.ceil(remaining / 60); /* Convert seconds to minutes if it's more than or equal to 60 seconds */
                isMinutes = true;
            }
        }

        result.remaining = remaining;
        result.is_minutes = isMinutes;

        return result;
    }

    /**
     * Add notification to the status list
     *
     * @param {Object} notification
     * @param {Array} requestQueuedIds
     * @param {Array} requestCompletedIds
     * @param {String} requestId
     *
     * @returns {void}
     */
    function addNotification(
        notification,
        requestQueuedIds,
        requestCompletedIds,
        requestId
    ) {
        var noStatusMatched = true;

        for (const status of statuses()) {
            if (
                status.id === tempId &&
                checkArraysIntersection(
                    requestQueuedIds,
                    requestCompletedIds,
                    status.queuedIds
                ) &&
                status.user === notification.user
            ) {
                statuses.replace(status, notification);
                noStatusMatched = false;
                break;
            }
            if (status.id === requestId) {
                notification.user = status.user;
                statuses.replace(status, notification);
                noStatusMatched = false;
                break;
            }
        }

        if (noStatusMatched) {
            const index = statuses().findIndex(status => status.user === notification.user);

            if (index !== -1) {
                statuses.splice(index, 1, notification);
            } else {
                statuses.push(notification);
            }
            
            opened(true);
        }
    }
    
    function addKeywordAnalysisFailedStatus(currentUser, messageData = {})
    {
        var keywordAnalysisStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser.email;
        }),
            newStatus = {
                id: messageData.id,
                message: $t('Keyword analysis processing failed.'),
                progress: $t('%s / %s product/s')
                    .replace('%s', 1)
                    .replace('%s', 1),
                user: currentUser.email,
                percent: 0,
                status: 'Completed',
                queuedIds: messageData.queueIds,
                completedIds: messageData.completedIds,
                subType: messageData.subType,
                failed: 1,
                failedIds: messageData.completedIds,
                total: 1,
                runningIds: messageData.runningIds
        };

        if (keywordAnalysisStatus) {
            statuses.replace(keywordAnalysisStatus, newStatus);
        } else {
            statuses.push(newStatus);
            opened(true);
        }
    }

    /**
     * Add new keyword analysis status
     *
     * @param {Array} productIds
     * @param {Object} currentUser
     * @param {Object} data
     *
     * @returns {void}
     */
    function addNewKeywordAnalysisStatus(productIds, currentUser, data = {})
    {
        var initialLength = productIds.length;
        if (data) {
            initialLength = data.excludeMode ? data.total : initialLength;
        }
        var keywordAnalysisStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser.email;
        }),
            newStatus = {
                id: koTempId,
                message: $t('Keyword analysis currently in progress'),
                progress: $t('%s / %s product/s')
                    .replace('%s', 0)
                    .replace('%s', initialLength),
                user: currentUser.email,
                percent: 0,
                status: 'Pending',
                queuedIds: productIds,
                completedIds: [],
                subType: 2,
                total: initialLength
        };

        if (keywordAnalysisStatus) {
            statuses.replace(keywordAnalysisStatus, newStatus);
        } else {
            statuses.push(newStatus);
            opened(true);
        }
    }

    /**
     * Add new generate status
     *
     * @param {Array} productIds
     * @param {Object} currentUser
     * @param {Object} data
     *
     * @returns {void}
     */
    function addNewGenerateStatus(productIds, currentUser, data = {})
    {
        var initialLength = productIds.length;
        if (data) {
            initialLength = data.excludeMode ? data.total : initialLength;
        }
        var generateStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser.email;
        }),
            newStatus = {
                id: tempId,
                message: $t('Generating text...'),
                progress: $t('%s / %s product/s')
                    .replace('%s', 0)
                    .replace('%s', initialLength),
                user: currentUser.email,
                percent: 0,
                status: 'Pending',
                queuedIds: productIds,
                completedIds: [],
                subType: data.params.auto_optimize_keywords ? 1 : 0,
                total: initialLength
        };

        if (generateStatus) {
            statuses.replace(generateStatus, newStatus);
        } else {
            statuses.push(newStatus);
            opened(true);
        }
    }

    /**
     * Add new transfer status
     *
     * @param {Array} productIds
     * @param {Object} currentUser
     *
     * @returns {void}
     */
    function addNewTransferStatus(productIds, currentUser)
    {
        var transferStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser.email;
        }),
            newStatus = {
                id: transferTempId,
                message: $t('Transferring text...'),
                progress: $t('%s / %s product/s')
                    .replace('%s', 0)
                    .replace('%s', productIds.length),
                user: currentUser.email,
                percent: 0,
                status: 'Running',
                transfer: true,
                queuedIds: productIds,
                completedIds: []
        };

        if (transferStatus) {
            statuses.replace(transferStatus, newStatus);
        } else {
            statuses.push(newStatus);
            opened(true);
        }
    }

    /**
     * Update transfer status
     *
     * @param {Array} productIds
     * @param {Object} currentUser
     * @param {Array} queueIds
     * @param {Array} completedIds
     * @param {Object} continueParams
     *
     * @returns {void}
     */
    function updateTransferStatus(productIds, currentUser, queueIds, completedIds, continueParams = {})
    {
        var transferStatus = ko.utils.arrayFirst(statuses(), function (status) {
                return status.user === currentUser;
        }),
            message = $t('Done transferring text for your selected products.'),
            progress = $t('%s / %s product/s')
                .replace('%s', completedIds.length)
                .replace('%s', productIds.length),
            percent = Math.floor(completedIds.length / productIds.length * 100),
            status = 'Completed',
            _queuedIds = [],
            _completedIds = [...productIds];

        if (completedIds.length !== productIds.length) {
            if (queueIds.length == 0) {
                message = $t('Cancelled');
                status = 'Completed';
                _queuedIds = [...queueIds];
                _completedIds = [...completedIds];
            } else {
                message = $t('Transferring text...');
                status = 'Running';
                _queuedIds = [...queueIds];
                _completedIds = [...completedIds];
            }
        }

        var newStatus = {
            id: transferTempId,
            message: message,
            progress: progress,
            user: currentUser,
            percent: percent,
            status: status,
            transfer: true,
            queuedIds: _queuedIds,
            completedIds: _completedIds,
            continueParams: continueParams
        };

        if (transferStatus) {
            statuses.replace(transferStatus, newStatus);
        } else {
            statuses.push(newStatus);
            opened(true);
        }
    }

    /**
     * Process the notification
     *
     * @param {Object} messageEntry
     * @returns {void}
     */
    function processNotification(messageEntry)
    {
        var requestId = messageEntry.encodedMsg.id,
            requestStatus = messageEntry.encodedMsg.status,
            requestEstimatedEndTime = messageEntry.encodedMsg.estimatedEndTime,
            requestQueuedIds = messageEntry.encodedMsg.queuedIds,
            requestRunningIds = messageEntry.encodedMsg.runningIds,
            requestCompletedIds = messageEntry.encodedMsg.completedIds,
            requestCompleted = messageEntry.encodedMsg.completed,
            requestTotal = messageEntry.encodedMsg.total,
            requestUserName = messageEntry.encodedMsg.userName,
            requestSubType = messageEntry.encodedMsg.subType,
            requestFailed = messageEntry.encodedMsg.failed,
            requestFailedIds = messageEntry.encodedMsg.failedIds,
            remaining = {},
            message = '',
            requestData = {},
            completed = requestCompleted !== requestFailed ? requestCompleted - requestFailed : requestCompleted;

        remaining = calculateRemainingTime(requestEstimatedEndTime);
        message = getMessage(messageEntry.encodedMsg, remaining);
        requestData = {
            id: requestId,
            message: message,
            progress: $t('%s / %s product/s').replace('%s', completed).replace('%s', requestTotal),
            percent: Math.floor(requestCompleted / requestTotal * 100),
            status: requestStatus,
            queuedIds: requestQueuedIds,
            runningIds: requestRunningIds,
            completedIds: requestCompletedIds,
            user: requestUserName,
            subType: requestSubType,
            failed: requestFailed,
            failedIds: requestFailedIds,
            total: requestTotal
        };

        addNotification(
            requestData,
            requestQueuedIds,
            requestCompletedIds,
            requestId
        );
    }

    /**
     * Add stream status
     *
     * @param {Object} messageEntry
     * @returns {void}
     */
    function addStreamStatus(
        messageEntry
    ) {
        processNotification(messageEntry);
    }

    return {
        statuses: statuses,
        tempId: tempId,
        transferTempId: transferTempId,
        koTempId: koTempId,
        opened: opened,
        addNewGenerateStatus: addNewGenerateStatus,
        addStreamStatus: addStreamStatus,
        addNewTransferStatus: addNewTransferStatus,
        updateTransferStatus: updateTransferStatus,
        addNewKeywordAnalysisStatus: addNewKeywordAnalysisStatus,
        addKeywordAnalysisFailedStatus: addKeywordAnalysisFailedStatus
    };
});
