/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/grid/upload-images',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/gallery',
    'WriteTextAI_WriteTextAI/js/model/grid/listing',
    'WriteTextAI_WriteTextAI/js/model/grid/invalid-image-popup'
], function (
    $,
    _,
    uploadImagesModel,
    referenceProducts,
    errorMessagesModel,
    galleryModel,
    listingModel,
    gridInvalidImagePopupModel
) {
    'use strict';

    /**
     * Main function for uploading images
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Boolean} needKeywordsOptimization - Whether keywords optimization is needed.
     *
     * @returns {void}
     */
    function uploadImages(self, action, data, settings, needKeywordsOptimization = false)
    {
        $('.wtai-upload-image-loader').modal('openModal');
        uploadImagesModel.failedThumbnails([]);
        uploadImagesModel.failedProducts([]);
        uploadImagesModel.successThumbnails([]);
        uploadThumbnails(self, action, data, settings, 0, needKeywordsOptimization);
    }

    /**
     * Upload thumbnails
     *
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Number} index - Index of the current chunk.
     * @param {Boolean} needKeywordsOptimization - Whether keywords optimization is needed.
     *
     * @returns {void}
     */
    function uploadThumbnails(self, action, data, settings, index = 0, needKeywordsOptimization = false)
    {
        var textTypes = [
            'page_title',
            'page_description',
            'product_description',
            'short_product_description',
            'open_graph'
        ];
        var selections = {};
        var endIndex = Math.min(index + 10, data.selected.length);
        var thumbnailAttrSelected = textTypes.some(textType => settings.fields.includes(textType)) &&
            settings.attributes &&
            settings.attributes.includes('thumbnail');

        if ((thumbnailAttrSelected || needKeywordsOptimization) && index < data.selected.length) {
            $.ajax({
                url: self.uploadThumbnailsUrl,
                type: 'POST',
                data: selections,
                data: {
                    selected: data.selected.slice(index, endIndex),
                    store_id: referenceProducts.storeId()
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        $('.wtai-upload-image-loader').modal('closeModal');
                        $('.wtai-upload-image-loader-ideas').modal('closeModal');
                        $('.modals-overlay.wtai-upload-image-loader-overlay').remove();
                        errorMessagesModel.messages.push(response.message);

                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                    } else {
                        uploadImagesModel.failedThumbnails.push(...response.failed_thumbnails);
                        uploadImagesModel.failedProducts.push(...response.failed_products);
                        uploadImagesModel.successThumbnails.push(...response.success_thumbnails);

                        /** Upload next chunk */
                        uploadThumbnails(self, action, data, settings, endIndex, needKeywordsOptimization);
                    }
                }
            });
        } else {
            getImages(self, action, data, settings);
        }
    }

    /**
     * Prepare alt text images for upload (batch upload)
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Array} productImages - Array of images.
     *
     * @returns {void}
     */
    function uploadAltTextImages(self, action, data, settings, productImages)
    {
        var ids = listingModel.ids(); /** Get ids of products in grid with/without filter */
        var selectedProductImages = Object.keys(productImages)
            .filter(productId => {
                if (data.excludeMode) {
                    return ids.includes(productId) && !data.excluded.includes(productId);
                } else {
                    return ids.includes(productId) && data.selected.includes(productId);
                }
            })
            .flatMap(productId => productImages[productId].split(',').map(image => ({
                image: image,
                entityId: productId
            })));

        /** Split images into chunks of 10 */
        var imageChunks = [];
        for (var i = 0; i < selectedProductImages.length; i += 10) {
            imageChunks.push(selectedProductImages.slice(i, i + 10));
        }

        galleryModel.productsUploading(data.selected);
        uploadImagesModel.failedImages([]);
        uploadImagesModel.failedImageIds([]);
        uploadImagesModel.successImages([]);

        /** Start uploading the first chunk */
        uploadRequest(self, action, data, settings, imageChunks, 0);
    }
    
    /**
     * AJAX request for uploading alt text images
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Array} imageChunks - Array of image chunks.
     * @param {Number} index - Index of the current chunk.
     *
     * @returns {void}
     */
    function uploadRequest(self, action, data, settings, imageChunks, index)
    {
        if (settings.fields.includes('alt_text') &&
            index < imageChunks.length
        ) {
            $.ajax({
                url: self.uploadImagesUrl,
                type: 'POST',
                data: {
                    images: imageChunks[index],
                    store_id: referenceProducts.storeId(),
                    uploaded_images: uploadImagesModel.successThumbnails()
                },
                dataType: 'json',
                showLoader: false
            }).done(function (response) {
                if (!response.success) {
                    $('.wtai-upload-image-loader').modal('closeModal');
                    $('.wtai-upload-image-loader-ideas').modal('closeModal');
                    $('.modals-overlay.wtai-upload-image-loader-overlay').remove();
                    errorMessagesModel.messages.push(response.message);
                    
                    if (response.unauthorized) {
                        window.location.href = response.login_url;
                    }
                } else {
                    uploadImagesModel.failedImages.push(...response.failed_images);
                    uploadImagesModel.failedImageIds.push(...response.failed_image_ids);
                    uploadImagesModel.successImages.push(...response.success_images);

                    /** Upload next chunk */
                    uploadRequest(self, action, data, settings, imageChunks, index + 1);
                }
            });
        } else {
            galleryModel.productsUploading([]);
            $('.wtai-upload-image-loader').modal('closeModal');
            $('.wtai-upload-image-loader-ideas').modal('closeModal');
            $('.modals-overlay.wtai-upload-image-loader-overlay').remove();

            data.params.uploaded_images = [...uploadImagesModel.successImages(), ...uploadImagesModel.successThumbnails()];
            if (validateInvalidImages()) {
                self.bulkGenerateRequest(self, action, data, settings);
            } else {
                gridInvalidImagePopupModel.data(data);
                gridInvalidImagePopupModel.settings(settings);
                $('.wtai-grid-invalid-image-popup').modal('openModal');
            }
        }
    }

    /**
     * Validate invalid images
     */
    function validateInvalidImages()
    {
        var failedImages = uploadImagesModel.failedImages(),
            failedThumbnails = uploadImagesModel.failedThumbnails();

        return failedThumbnails.length === 0 && failedImages.length === 0;
    }

    /**
     * Get alt text images to upload
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function getImages(self, action, data, settings)
    {
        var itemsType = '',
            selections = {};

        _.extend(data.params, settings || {});

        itemsType = data.excludeMode ? 'excluded' : 'selected';

        /** Fixes for select all not actually selecting all */
        if (data && data.excludeMode) {
            data.selected = listingModel.ids().filter(id => !data.excluded.includes(id));
            selections.selected = listingModel.ids().filter(id => !data.excluded.includes(id));
        }

        selections[itemsType] = data[itemsType];

        if (!selections[itemsType].length) {
            selections[itemsType] = false;
        }

        _.extend(selections, data.params || {});

        /* set default store filter */
        if (!selections['filters']['store_id']) {
            selections['filters']['store_id'] = self.defaultStoreId;
        }

        $.ajax({
            url: self.getImagesUrl,
            type: 'POST',
            data: selections,
            dataType: 'json',
            showLoader: false,
            success: function (response) {
                if (!response.success) {
                    errorMessagesModel.messages.push(response.message);
                    if (response.unauthorized) {
                        window.location.href = response.login_url;
                    }
                    if (response.reload) {
                        self.gridReload();
                    }
                } else {
                    uploadAltTextImages(self, action, data, settings, response.images);
                }
            }
        });
    }

    return {
        uploadImages: uploadImages,
        uploadThumbnails: uploadThumbnails,
        uploadAltTextImages: uploadAltTextImages,
        uploadRequest: uploadRequest,
        validateInvalidImages: validateInvalidImages,
        getImages: getImages
    };
});
