/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/grid/listing',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/transfer',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications'
], function (
    $,
    _,
    listingModel,
    errorMessagesModel,
    transfer,
    notifications
) {
    'use strict';
    
    /**
     * Check if transferable
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function transferable(self, action, data, settings)
    {
        var itemsType = '',
            selections = {};

        _.extend(data.params, settings || {});

        itemsType = data.excludeMode ? 'excluded' : 'selected';

        /** Fixes for select all not actually selecting all */
        if (data && data.excludeMode) {
            data.selected = listingModel.ids().filter(id => !data.excluded.includes(id));
            selections.selected = listingModel.ids().filter(id => !data.excluded.includes(id));
        }

        selections[itemsType] = data[itemsType];

        if (!selections[itemsType].length) {
            selections[itemsType] = false;
        }

        _.extend(selections, data.params || {});

        /* set default store filter */
        if (!selections['filters']['store_id']) {
            selections['filters']['store_id'] = self.defaultStoreId;
        }
                    
        notifications.addNewTransferStatus(
            data.selected,
            listingModel.currentUser(),
            data
        );
        notifications.opened(true);

        $.ajax({
            url: self.transferableUrl,
            type: 'POST',
            data: selections,
            dataType: 'json',
            showLoader: false,
            success: function (response) {
                if (!response.success) {
                    errorMessagesModel.messages.push(response.message);
                    notifications.statuses.remove(function (status) {
                        return status.id === notifications.transferTempId;
                    });
                    if (response.unauthorized) {
                        window.location.href = response.login_url;
                    }
                    if (response.reload) {
                        self.gridReload();
                    }
                } else {
                    transfer.queueIds(data.selected);
                    transfer.completedIds([]);
                    transfer.cancelBulkTransfer(false);
                    saveBulkTransferProgress(self, data, settings);
                    bulkTransfer(self, action, data, settings, 0);
                }
            }
        });
    }

    /**
     * Bulk transfer request
     *
     * @param {Object} self - Self data.
     * @param {Object} action - Action data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     * @param {Number} index - Index of the current chunk.
     *
     * @returns {void}
     */
    function bulkTransfer(self, action, data, settings, index)
    {
        var itemsType = '',
            selections = {};

        if (index < data.selected.length) {
            _.extend(data.params, settings || {});

            itemsType = data.excludeMode ? 'excluded' : 'selected';

            selections[itemsType] = data[itemsType];

            if (!selections[itemsType].length) {
                selections[itemsType] = false;
            }

            _.extend(selections, data.params || {});

            /* set default store filter */
            if (!selections['filters']['store_id']) {
                selections['filters']['store_id'] = self.defaultStoreId;
            }
            
            let productIds = [...data.selected];
            let completedIds = [...transfer.completedIds()];
            let queueIds = transfer.queueIds().filter(item => item !== productIds[index]);

            completedIds.push(productIds[index]);

            transfer.completedIds(completedIds);
            transfer.queueIds(queueIds);

            selections.selected = [data.selected[index]];

            $.ajax({
                url: self.bulkTransferUrl,
                type: 'POST',
                data: selections,
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                        if (response.reload) {
                            self.gridReload();
                        }
                    }
                    if (transfer.cancelBulkTransfer() === false) {
                        notifications.updateTransferStatus(
                            productIds,
                            listingModel.currentUser().email,
                            queueIds,
                            completedIds
                        );
                        saveBulkTransferProgress(self, data, settings);
                        bulkTransfer(self, action, data, settings, index + 1);
                    } else {
                        notifications.updateTransferStatus(
                            productIds,
                            listingModel.currentUser().email,
                            [],
                            completedIds
                        );
                    }
                }
            });
        } else {
            self.gridReload();
        }
    }

    /**
     * Save bulk transfer progress
     *
     * @param {Object} self - Self data.
     * @param {Object} data - Selections data.
     * @param {Object} settings - Form settings.
     *
     * @returns {void}
     */
    function saveBulkTransferProgress(self, data, settings)
    {
        $.ajax({
            url: self.saveTransferProgressUrl,
            type: 'POST',
            data: {
                queue_ids: transfer.queueIds(),
                completed_ids: transfer.completedIds(),
                store_id: data.params.filters['store_id'] ?? self.defaultStoreId,
                stores: settings.stores,
                fields: settings.fields,
                user: listingModel.currentUser().email
            },
            dataType: 'json',
            showLoader: false,
            success: function (response) {
                if (!response.success) {
                    console.log(response.message);
                }
            }
        });
    }

    /**
     * Continue bulk transfer
     *
     * @param {Object} self - Self data.
     *
     * @returns {void}
     */
    function continueBulkTransfer(self)
    {
        try {
            notifications.statuses().forEach(status => {
                if (status.status === self.statusRunning &&
                    status.user === listingModel.currentUser().email &&
                    status.transfer === true &&
                    status.queuedIds.length > 0
                ) {
                    let action = {},
                        data = {
                            excludeMode: false,
                            excluded: [],
                            params: {
                                fields: status.continueParams.fields,
                                stores: status.continueParams.stores,
                                namespace: "wtai_products_grid_listing",
                                filters: {
                                    store_id: status.continueParams.store_filter
                                }
                            },
                            /** always set completed first so the index is correct */
                            selected: [...status.completedIds, ...status.queuedIds],
                            total: status.queuedIds.length + status.completedIds.length
                    },
                        settings = {
                            stores: status.continueParams.stores,
                            fields: status.continueParams.fields
                    },
                        index = status.completedIds.length;
                        
                    transfer.completedIds(status.completedIds);
                    transfer.queueIds(status.queuedIds);
                    bulkTransfer(self, action, data, settings, index);
                }
            });
        } catch (e) {
            return;
        }
    }

    return {
        transferable: transferable,
        bulkTransfer: bulkTransfer,
        saveBulkTransferProgress: saveBulkTransferProgress,
        continueBulkTransfer: continueBulkTransfer
    };
});
